function creator( services ){
    function normalize( inVal, inMin, inMax ) {
        var inRange = inMax - inMin || 1;
        return Math.max(0, Math.min(1, (inVal - inMin) / inRange));
    }

    function remap( inVal, inMin, inMax, outMin, outMax ) {
        return outMin + (normalize( inVal, inMin, inMax ) * (outMax - outMin));
    }

    function evalWinLosePriority( perception, perceptionDelta ) {
        var oppProgressMax = Math.max( perception.oppProgress, perception.oppProgress + perception.oppProgressPotential );

        // Pushes the opponent to the winning level?
        if( oppProgressMax >= 1 ) {
            // A tie situation?
            if( perception.myProgress >= 1 )
                return 0.8;

            // If the player's already going to win prior to this move, don't affect the scale...
            var oppProgressPrev     = perception.oppProgress - perceptionDelta.oppProgress,
                oppProgressMaxPrev  = Math.max( oppProgressPrev, oppProgressPrev + perception.oppProgressPotential - perceptionDelta.oppProgressPotential );
            if( oppProgressMaxPrev >= 1 )
                return 0;

            // Pushing the opponent progress to completion is worse than pushing the progress combination to completion
            return perception.oppProgress >= 1 ? -1 : -0.25;
        }

        // Neither a win-lose proposition? No priority impact
        if( perception.myProgress < 1 )
            return 0;

        // If we get here, it's a winning proposition
        return 1;
    }

    return {
        id:         "__ai_buzz",
        name:       "Buzz",
        portrait:   "portrait_teddy",
        level:      1,
        deck:       [
            "normal"
        ],
        brain:      "app/game/ai/brainsimple/brainsimple",
        recipe:   {
            actionAssessDepthMax:       2,
            decisionActionWeightMin:    0.2,
            decisionActionWeightRange:  0.1,

            onPreviewGetRegionValue: function( regionIdx, regionValuesAvailable, regionSpecific ) {
                // Teddy "knows" about regions with one specific value...
                if( regionSpecific && regionValuesAvailable.length == 1 )
                    return regionValuesAvailable[0];

                // For all the others, he's desperately hoping it's going to a 2-tribute region...
                return { credits: -1, tributes: 2 };
            },
            goals: {
                attack: {
                    calcGoalPriority: function( hints, perception ){
                        // The bigger the difference in progress, the more the AI will attack. As we get closer to the end of the game, the perception of what's dangerous progress increases
                        var progressDangerous   = hints.progressDangerous * 2,
                            dangerContrib       = normalize(perception.myProgress + perception.myProgressPotential - perception.oppProgress - perception.oppProgressPotential, 0, -progressDangerous);

                        return 0.2 + (0.6 * dangerContrib);
                    },
                    calcActionPriority: function( hints, perception, perceptionDelta  ) {
                        // +ve personal progress and -ve opponent progress contributes to a action's priority (up to the "dangerous"  progress range)
                        var progressDangerous           = hints.progressDangerous * 2;
                        var progressContrib             = remap( perceptionDelta.myProgress - perceptionDelta.oppProgress, -progressDangerous, progressDangerous, -1, 1);
                        var potentialContrib            = remap( perception.myProgressPotential - perception.oppProgressPotential, -progressDangerous, progressDangerous, -1, 1 );

                        return (evalWinLosePriority(perception, perceptionDelta) + (progressContrib) + (potentialContrib));
                    }
                },
                getGems: {
                    calcGoalPriority: function( hints, perception ){
                        // The weaker its income strength, the stronger the priority...
                        return 0.1 + (0.2 * normalize( perception.myIncomeStrength, 0, -4 ));
                    },
                    calcActionPriority: function( hints, perception, perceptionDelta  ) {
                        // The bigger our income strength increase, the better...
                        var incomeStrengthContrib;

                        // Negative income strength? Negative contribution. Otherwise positive contribution...
                        if( perceptionDelta.myIncomeStrength < 0 )
                            incomeStrengthContrib = -normalize( perceptionDelta.myIncomeStrength, 0, -1 );
                        else
                            incomeStrengthContrib = normalize( perceptionDelta.myIncomeStrength, 0, 2 );

                        return evalWinLosePriority(perception, perceptionDelta) + incomeStrengthContrib;
                    }
                },
                buildArmy: {
                    calcGoalPriority: function( hints, perception ){
                        // The more negative the troop strength, the stronger the need
                        return normalize( perception.myTroopStrength, 2, -12 );
                    },
                    calcActionPriority: function( hints, perception, perceptionDelta  ) {
                        // Negative troop strength saps priority. Positive troop strength boosts priority
                        var troopStrengthContrib;
                        if( perceptionDelta.myTroopStrength < 0 )
                            troopStrengthContrib = -normalize( perceptionDelta.myTroopStrength, 0, -1 );
                        else
                            // Positive troop strength boosts its priority...
                            troopStrengthContrib = normalize( perceptionDelta.myTroopStrength, 0, 3 );

                        return (evalWinLosePriority(perception, perceptionDelta) + troopStrengthContrib);
                    }
                },
                conquer: {
                    calcGoalPriority: function( hints, perception ) {
                        // Conquering is what this AI is about...
                        return 1 * normalize( perception.myMapAttackStrength, 1, 0.2 );
                    },
                    calcActionPriority: function( hints, perception, perceptionDelta  ) {
                        // Negative map strength saps priority. Positive map strength boosts priority
                        var mapStrengthContrib;
                        if( perceptionDelta.myMapAttackStrength < 0 )
                            mapStrengthContrib = -normalize( perceptionDelta.myMapAttackStrength, 0, -hints.regionStrengthAvg * 0.5 );
                        else
                            mapStrengthContrib = normalize( perceptionDelta.myMapAttackStrength, 0, hints.regionStrengthAvg * 1.2 );

                        // The bigger the map strength contribution, the bigger its impact
                        return (evalWinLosePriority(perception, perceptionDelta) + mapStrengthContrib);
                    }
                },
                explore: {
                    calcGoalPriority: function( hints, perception ) {
                        // Teddy likes to explore
                        return 0.2 * normalize( perception.myMapExploration, 1, 0 );
                    },
                    calcActionPriority: function( hints, perception, perceptionDelta  ) {
                        // Negative map strength saps priority. Positive map strength boosts priority
                        var exploreContrib = remap( perceptionDelta.myMapExploration, -hints.regionStrengthAvg, hints.regionStrengthAvg, -1, 1 );

                        // The bigger the map strength contribution, the bigger its impact
                        return (evalWinLosePriority(perception, perceptionDelta) + exploreContrib);
                    }
                },
                defend: {
                    calcGoalPriority: function( hints, perception ) {
                        var progressDangerous = hints.progressDangerous * 2;

                        // The bigger the gap, the more serious the defense gets...
                        var potentialDifContrib = normalize( perception.oppProgressPotential - perception.myProgressPotential, 0, progressDangerous );

                        // This goal's priority is the greater of the two...
                        return 0.4 * potentialDifContrib;
                    },
                    calcActionPriority: function( hints, perception, perceptionDelta  ) {
                        var progressDangerous   = hints.progressDangerous * 2,
                            oppProgressContrib  = remap( perceptionDelta.oppProgress, -progressDangerous, progressDangerous, -1, 1 );

                        // The action which reduces the opponent progress or potential progress the most is the winner...
                        return (evalWinLosePriority(perception, perceptionDelta) + oppProgressContrib);
                    }
                }
            }
        }
    };
}

creator;