function creator( services ){
    function normalize( inVal, inMin, inMax ) {
        var inRange = inMax - inMin || 1;
        return Math.max(0, Math.min(1, (inVal - inMin) / inRange));
    }

    function remap( inVal, inMin, inMax, outMin, outMax ) {
        return outMin + (normalize( inVal, inMin, inMax ) * (outMax - outMin));
    }

    function evalWinLosePriority( perception, perceptionDelta ) {
        var oppProgressMax = Math.max( perception.oppProgress, perception.oppProgress + perception.oppProgressPotential );

        // Pushes the opponent to the winning level?
        if( oppProgressMax >= 1 ) {
            // A tie situation?
            if( perception.myProgress >= 1 )
                return 0.8;

            // If the player's already going to win prior to this move, don't affect the scale...
            var oppProgressPrev     = perception.oppProgress - perceptionDelta.oppProgress,
                oppProgressMaxPrev  = Math.max( oppProgressPrev, oppProgressPrev + perception.oppProgressPotential - perceptionDelta.oppProgressPotential );
            if( oppProgressMaxPrev >= 1 )
                return 0;

            // Pushing the opponent progress to completion is worse than pushing the progress combination to completion
            return perception.oppProgress >= 1 ? -1 : -0.25;
        }

        // Neither a win-lose proposition? No priority impact
        if( perception.myProgress < 1 )
            return 0;

        // If we get here, it's a winning proposition
        return 1;
    }

    function evalCostModifier( perceptionDelta ) {
        return 1 + (perceptionDelta.myCredits * 0.05);
    }

    var kCostPotentialImpactCostMin     = 0;
    var kCostPotentialImpactLevelMin    = 0.5;
    var kCostPotentialImpactMult        = 10;
    var kCostPotentialImpactExp         = 1;

    function evalProgressPotentialEffective( progressPotential, progressPotentialCost ) {
        // If the cost exceeds 100%, the progress potential is scaled back to fit teh 100% potential cost scale...
        if( progressPotentialCost > 1 ) {
            progressPotential       = progressPotential / progressPotentialCost;
            progressPotentialCost   = 1;
        }

        // We want the effective potential multiplier to drop asynptotically towards kCostPotentialImpactLevelMin starting at kCostPotentialImpactCostMin
        // The formula is:
        //   F(c) = kCostPotentialImpactLevelMin +                       (1 - kCostPotentialImpactLevelMin)
        //                                         -------------------------------------------------------------------------------
        //                                           ( MULT * (Math.max(0, c - kCostPotentialImpactCostMin) + (1/MULT)) ) ^ EXP
        var effectiveMult   = kCostPotentialImpactLevelMin +  ( (1 - kCostPotentialImpactLevelMin) / Math.pow(kCostPotentialImpactMult * (Math.max(0,progressPotentialCost - kCostPotentialImpactCostMin) + (1/kCostPotentialImpactMult)), kCostPotentialImpactExp) );

        return effectiveMult * progressPotential;
    }

    return {
        id:         "__ai_om",
        name:       "Om",
        portrait:   "portrait_sherlock",
        level:      3,
        deck:       [
            "normal"
        ],
        brain:      "app/game/ai/brainsimple/brainsimple",
        recipe:   {
            actionAssessDepthMax:       2,
            goalsMax:                   4,
            decisionActionWeightMin:    0.25,
            decisionActionWeightRange:  0.01,

            onPreviewGetRegionValue: function( regionIdx, regionValuesAvailable, regionSpecific ) {
                if( regionValuesAvailable.length == 1 )
                    return regionValuesAvailable[0];

                // Base the region value on what he knows is still available...
                if( !regionSpecific ) {
                    // Select the median value...
                    regionValuesAvailable = regionValuesAvailable.slice().sort( function( entry1, entry2 ) {
                        var compareResult = entry2.tributes - entry1.tributes;
                        if( compareResult == 0 )
                            return entry2.credits - entry1.credits;

                        return compareResult;
                    } );

                    return regionValuesAvailable[ Math.floor((regionValuesAvailable.length + Math.random()) * 2 / 3) ];
                }

                // For region with specific values, he "remembers" that... but is optimistic he'll get the the highest value
                var regionsAvailableNum = regionValuesAvailable.length,
                    topValue            = regionValuesAvailable[0];
                for( var regionAvailableIdx = 1; regionAvailableIdx < regionsAvailableNum; regionAvailableIdx++ ) {
                    var valueCurr = regionValuesAvailable[regionAvailableIdx];
                    if( valueCurr.tributes < topValue.tributes )
                        continue;

                    if( valueCurr.credits < topValue.credits )
                        topValue = valueCurr;
                }

                return topValue;
            },
            goals: {
                attack: {
                    calcGoalPriority: function( hints, perception ){
                        // The bigger the difference in progress, the more the AI will attack. As we get closer to the end of the game, the perception of what's dangerous progress increases
                        var progressDangerous   = Math.min( hints.progressDangerous, 0.5 * (1 - perception.oppProgress)),
                            dangerContrib       = normalize(perception.myProgress + perception.myProgressPotential - perception.oppProgress - perception.oppProgressPotential, 0, -progressDangerous);

                        return 0.6 + (dangerContrib * 0.4);
                    },
                    calcActionPriority: function( hints, perception, perceptionDelta  ) {
                        // +ve personal progress and -ve opponent progress contributes to a action's priority (up to the "dangerous"  progress range)
                        var progressDangerous           = Math.min( hints.progressDangerous, 0.5 * (1 - perception.oppProgress));
                        var progressContrib             = remap( perceptionDelta.myProgress - perceptionDelta.oppProgress, -progressDangerous, progressDangerous, -1, 1);
                        var myPotentialEffectiveDelta   = evalProgressPotentialEffective( perception.myProgressPotential, perception.myProgressPotentialCost ) - evalProgressPotentialEffective( perception.myProgressPotential - perceptionDelta.myProgressPotential, perception.myProgressPotentialCost - perceptionDelta.myProgressPotentialCost),
                            oppPotentialEffectiveDelta  = evalProgressPotentialEffective( perception.oppProgressPotential, perception.oppProgressPotentialCost ) - evalProgressPotentialEffective( perception.oppProgressPotential - perceptionDelta.oppProgressPotential, perception.oppProgressPotentialCost - perceptionDelta.oppProgressPotentialCost),
                            potentialContrib            = remap( myPotentialEffectiveDelta - oppPotentialEffectiveDelta, -progressDangerous, progressDangerous, -1, 1 );

                        return evalCostModifier(perceptionDelta)
                            * (evalWinLosePriority(perception, perceptionDelta) + (1.8 * progressContrib) + (potentialContrib));
                    }
                },
                getGems: {
                    calcGoalPriority: function( hints, perception ){
                        // If weaker our income strength, the stronger the priority...
                        return 0.1 + (0.5 * normalize( perception.myIncomeStrength, 0, -2 ));
                    },
                    calcActionPriority: function( hints, perception, perceptionDelta  ) {
                        // The bigger our income strength increase, the better...
                        var incomeStrengthContrib;

                        // Negative income strength? Negative contribution. Otherwise positive contribution...
                        if( perceptionDelta.myIncomeStrength < 0 )
                            incomeStrengthContrib = -normalize( perceptionDelta.myIncomeStrength, 0, -1 );
                        else
                            incomeStrengthContrib = normalize( perceptionDelta.myIncomeStrength, 0, 2 );

                        return evalCostModifier(perceptionDelta)
                            * (evalWinLosePriority(perception, perceptionDelta) + incomeStrengthContrib);
                    }
                },
                buildArmy: {
                    calcGoalPriority: function( hints, perception ){
                        // The more negative the troop strength, the stronger the need
                        return 0.8 * normalize( perception.myTroopStrength, 2, -4 );
                    },
                    calcActionPriority: function( hints, perception, perceptionDelta  ) {
                        // Negative troop strength saps priority. Positive troop strength boosts priority
                        var troopStrengthContrib;
                        if( perceptionDelta.myTroopStrength < 0 )
                            troopStrengthContrib = -normalize( perceptionDelta.myTroopStrength, 0, -1 );
                        else
                            // Positive troop strength boosts its priority...
                            troopStrengthContrib = normalize( perceptionDelta.myTroopStrength, 0, 1.5 );

                        return evalCostModifier(perceptionDelta) *
                            (evalWinLosePriority(perception, perceptionDelta) + troopStrengthContrib);
                    }
                },
                conquer: {
                    calcGoalPriority: function( hints, perception ) {
                        // For the AI, having only 40% domination makes conquering the top priority...
                        return 0.2 + (0.4 * normalize( perception.myMapAttackStrength, 0.8, 0.4 ));
                    },
                    calcActionPriority: function( hints, perception, perceptionDelta  ) {
                        // Negative map strength saps priority. Positive map strength boosts priority
                        var mapStrengthContrib;
                        if( perceptionDelta.myMapAttackStrength < 0 )
                            mapStrengthContrib = -normalize( perceptionDelta.myMapAttackStrength, 0, -hints.regionStrengthAvg * 0.5 );
                        else
                            mapStrengthContrib = normalize( perceptionDelta.myMapAttackStrength, 0, hints.regionStrengthAvg * 1.2 );

                        // The bigger the map strength contribution, the bigger its impact
                        return evalCostModifier(perceptionDelta) *
                            (evalWinLosePriority(perception, perceptionDelta) + mapStrengthContrib);
                    }
                },
                defend: {
                    calcGoalPriority: function( hints, perception ) {
                        var progressDangerous = Math.min( hints.progressDangerous, 0.5 * (1 - perception.oppProgress) );

                        // The bigger the gap, the more serious the defense gets...
                        var potentialDifContrib = normalize( Math.max(perception.oppProgressPotentialStrength - perception.myProgressPotentialStrength,perception.oppProgressPotential - perception.myProgressPotential), 0, progressDangerous );

                        // This goal's priority is the greater of the two...
                        return (0.6 * potentialDifContrib);
                    },
                    calcActionPriority: function( hints, perception, perceptionDelta  ) {
                        // The potential decrease is either the actual progress potential drop... or the progress potential strength drop (weigh more towards the actual potential decrease)
                        var oppPotentialEffectiveDelta  = evalProgressPotentialEffective( perception.oppProgressPotential, perception.oppProgressPotentialCost ) - evalProgressPotentialEffective( perception.oppProgressPotential - perceptionDelta.oppProgressPotential, perception.oppProgressPotentialCost - perceptionDelta.oppProgressPotentialCost),
                            progressDangerous           = Math.min( hints.progressDangerous, 0.5 * (1 - perception.oppProgress) );

                        // The action which reduces the opponent progress or potential progress the most is the winner...
                        return evalCostModifier(perceptionDelta) *
                            (evalWinLosePriority(perception, perceptionDelta) + remap(perceptionDelta.oppProgress + oppPotentialEffectiveDelta, progressDangerous * 0.5, -progressDangerous * 0.5, -1, 1));
                    }
                }
            }
        }
    };
}

creator;