function creator( services ){
    function normalize( inVal, inMin, inMax ) {
        var inRange = inMax - inMin || 1;
        return Math.max(0, Math.min(1, (inVal - inMin) / inRange));
    }

    function remap( inVal, inMin, inMax, outMin, outMax ) {
        return outMin + (normalize( inVal, inMin, inMax ) * (outMax - outMin));
    }

    function evalWinLosePriority( perception, perceptionDelta ) {
        var oppProgressMax = Math.max( perception.oppProgress, perception.oppProgress + perception.oppProgressPotential );

        // Pushes the opponent to the winning level?
        if( oppProgressMax >= 1 ) {
            // A tie situation?
            if( perception.myProgress >= 1 )
                return 0.8;

            // If the player's already going to win prior to this move, don't affect the scale...
            var oppProgressPrev     = perception.oppProgress - perceptionDelta.oppProgress,
                oppProgressMaxPrev  = Math.max( oppProgressPrev, oppProgressPrev + perception.oppProgressPotential - perceptionDelta.oppProgressPotential );
            if( oppProgressMaxPrev >= 1 )
                return 0;

            // Pushing the opponent progress to completion is worse than pushing the progress combination to completion
            return perception.oppProgress >= 1 ? -1 : -0.25;
        }

        // Neither a win-lose proposition? No priority impact
        if( perception.myProgress < 1 )
            return 0;

        // If we get here, it's a winning proposition
        return 1;
    }

    function evalCostModifier( perceptionDelta ) {
        return 1 + (perceptionDelta.myCredits * 0.05);
    }

    return {
        id:         "__ai_sparkx",
        name:       "Sparkx",
        portrait:   "portrait_joe",
        level:      2,
        deck:       [
            "normal"
        ],
        brain:      "app/game/ai/brainsimple/brainsimple",
        recipe:   {
            actionAssessDepthMax:       2,
            goalsMax:                   3,
            decisionActionWeightMin:    0.25,
            decisionActionWeightRange:  0.1,

            onPreviewGetRegionValue: function( regionIdx, regionValuesAvailable, regionSpecific ) {
                if( regionValuesAvailable.length == 1 )
                    return regionValuesAvailable[0];

                // Randomly select a region value for everything else...
                return regionValuesAvailable[ (Math.random() * regionValuesAvailable.length) >> 0 ];
            },
            goals: {
                attack: {
                    calcGoalPriority: function( hints, perception ){
                        // The bigger the difference in progress, the more the AI will attack. As we get closer to the end of the game, the perception of what's dangerous progress increases
                        var progressDangerous   = Math.min( hints.progressDangerous * 2, 1.5 * (1 - perception.oppProgress)),
                            dangerContrib       = normalize(perception.myProgress + perception.myProgressPotential - perception.oppProgress - perception.oppProgressPotential, 0, -progressDangerous);

                        return 0.2 + (dangerContrib * 0.8);
                    },
                    calcActionPriority: function( hints, perception, perceptionDelta  ) {
                        // +ve personal progress and -ve opponent progress contributes to a action's priority (up to the "dangerous"  progress range)
                        var progressDangerous           = Math.min( hints.progressDangerous, 0.5 * (1 - perception.oppProgress));
                        var progressContrib             = remap( perceptionDelta.myProgress - perceptionDelta.oppProgress, -progressDangerous, progressDangerous, -1, 1);
                        var potentialContrib            = remap( perceptionDelta.myProgressPotential - perceptionDelta.oppProgressPotential, -progressDangerous, progressDangerous, -1, 1 );

                        return evalCostModifier(perceptionDelta)
                            * (evalWinLosePriority(perception, perceptionDelta) + (1.8 * progressContrib) + (potentialContrib));
                    }
                },
                getGems: {
                    calcGoalPriority: function( hints, perception ){
                        // If weaker our income strength, the stronger the priority...
                        return 0.1 + (0.5 * normalize( perception.myIncomeStrength, 0, -2 ));
                    },
                    calcActionPriority: function( hints, perception, perceptionDelta  ) {
                        // The bigger our income strength increase, the better...
                        var incomeStrengthContrib;

                        // Negative income strength? Negative contribution. Otherwise positive contribution...
                        if( perceptionDelta.myIncomeStrength < 0 )
                            incomeStrengthContrib = -normalize( perceptionDelta.myIncomeStrength, 0, -1 );
                        else
                            incomeStrengthContrib = normalize( perceptionDelta.myIncomeStrength, 0, 2 );

                        return evalCostModifier(perceptionDelta)
                            * (evalWinLosePriority(perception, perceptionDelta) + incomeStrengthContrib);
                    }
                },
                buildArmy: {
                    calcGoalPriority: function( hints, perception ){
                        // The more negative the troop strength, the stronger the need
                        return normalize( perception.myTroopStrength, 2, -4 );
                    },
                    calcActionPriority: function( hints, perception, perceptionDelta  ) {
                        // Negative troop strength saps priority. Positive troop strength boosts priority
                        var troopStrengthContrib;
                        if( perceptionDelta.myTroopStrength < 0 )
                            troopStrengthContrib = -normalize( perceptionDelta.myTroopStrength, 0, -1 );
                        else
                        // Positive troop strength boosts its priority...
                            troopStrengthContrib = normalize( perceptionDelta.myTroopStrength, 0, 1.5 );

                        return evalCostModifier(perceptionDelta) *
                            (evalWinLosePriority(perception, perceptionDelta) + troopStrengthContrib);
                    }
                },
                conquer: {
                    calcGoalPriority: function( hints, perception ) {
                        // For the AI, conquering is its greatest priority
                        return 0.3 + (0.6 * normalize( perception.myMapAttackStrength, 0.7, 0.2 ));
                    },
                    calcActionPriority: function( hints, perception, perceptionDelta  ) {
                        // Negative map strength saps priority. Positive map strength boosts priority
                        var mapStrengthContrib;
                        if( perceptionDelta.myMapAttackStrength < 0 )
                            mapStrengthContrib = -normalize( perceptionDelta.myMapAttackStrength, 0, -hints.regionStrengthAvg * 0.5 );
                        else
                            mapStrengthContrib = normalize( perceptionDelta.myMapAttackStrength, 0, hints.regionStrengthAvg * 1.2 );

                        // The bigger the map strength contribution, the bigger its impact
                        return evalCostModifier(perceptionDelta) *
                            (evalWinLosePriority(perception, perceptionDelta) + mapStrengthContrib);
                    }
                },
                defend: {
                    calcGoalPriority: function( hints, perception ) {
                        var progressDangerous = Math.min( hints.progressDangerous, 0.5 * (1 - perception.oppProgress) ) * 0.8;

                        // The bigger the gap, the more serious the defense gets...
                        var potentialDifContrib = normalize( Math.max(perception.oppProgressPotentialStrength - perception.myProgressPotentialStrength,perception.oppProgressPotential - perception.myProgressPotential), 0, progressDangerous );

                        // This goal's priority is the greater of the two...
                        return 0.2 + (0.7 * potentialDifContrib);
                    },
                    calcActionPriority: function( hints, perception, perceptionDelta  ) {
                        // The potential decrease is either the actual progress potential drop... or the progress potential strength drop (weigh more towards the actual potential decrease)
                        var progressDangerous           = Math.min( hints.progressDangerous, 0.5 * (1 - perception.oppProgress) );

                        // The action which reduces the opponent progress or potential progress the most is the winner...
                        return evalCostModifier(perceptionDelta) *
                            (evalWinLosePriority(perception, perceptionDelta) + remap(perceptionDelta.oppProgress + perceptionDelta.oppProgressPotential, progressDangerous * 0.5, -progressDangerous * 0.5, -1, 1));
                    }
                }
            }
        }
    };
}

creator;