/**
 * User: ckhoo
 * Date: 11/02/14
 * Time: 11:38 AM
 *
 * Copyright 2012 Wappworks Studio
 */

"use strict";

var TrackerWebView = (function( document ){
    /**
     * Flurry metrics tracker
     * @constructor
     */
    function TrackerFlurry() {}
    TrackerFlurry.prototype = {
        config: null,
        paused: false,
        inited: false,

        /**
         * Initialize the tracker..
         *
         * @param config        {Object}
         *
         * @returns {Boolean}
         *
         * @param config.apiKey         {String}
         * @param [config.appVersion]   {String}
         * @param [config.debugLog]     {Boolean}
         */
        init: function( config ) {
            this.config = config;
            this.inited = ( FlurryAgent != null );

            if( !this.inited )
                return false;

            // If debug log is enabled, redirect the console log output to the page body...
            if( config.debugLog ) {
                var body    = document.getElementsByTagName('body')[0];
                console.log = function( msg ) {
                    var logElem = document.createElement("p");
                    logElem.innerText = msg;
                    body.appendChild( logElem );
                };
            }

            // Start Flurry...
            FlurryAgent.setDebugLogEnabled( config.debugLog || false );
            if( typeof config.appVersion == "string" )
                FlurryAgent.setAppVersion( config.appVersion );

            FlurryAgent.startSession( config.apiKey );
            return true;
        },

        /**
         * Shutdown the instance
         * @returns {TrackerFlurry}
         */
        deInit: function() {
            if( !this.inited )
                return this;

            FlurryAgent.endSession();
            return this;
        },

        /**
         * Set the user ID...
         *
         * @param userId    {String}
         *
         * @returns {TrackerFlurry}
         */
        setUserId: function( userId ) {
            if( !this.inited )
                return this;

            FlurryAgent.setUserId( userId );

            return this;
        },

        /**
         * Set the metrics paused state
         *
         * @param paused    {Boolean}
         *
         * @returns {TrackerFlurry}
         */
        setPaused: function( paused ) {
            if( !this.inited )
                return this;

            if( this.paused == paused )
                return this;

            this.paused = paused;
            if( paused )
                FlurryAgent.pauseSession();
            else
                FlurryAgent.startSession( this.config.apiKey );

            return this;
        },

        /**
         * Log an error
         *
         * @param errorName     {String}
         * @param errorMessage  {String}
         * @param lineNumber    {Number}
         *
         * @returns {TrackerFlurry}
         */
        logError: function( errorName, errorMessage, lineNumber ) {
            if( !this.inited )
                return this;

            FlurryAgent.logError( errorName, errorMessage, lineNumber );

            return this;
        },

        /**
         * Log a game event
         *
         * @param eventName     {String}
         * @param [eventParms]  {Object.<String, String>}
         *
         * @returns {TrackerFlurry}
         */
        logEvent: function( eventName, eventParms ) {
            if( !this.inited )
                return this;

            if( eventParms == null )
                FlurryAgent.logEvent( eventName );
            else
                FlurryAgent.logEvent( eventName, eventParms );

            return this;
        },

        /**
         * Start timed game event
         *
         * @param eventName     {String}
         * @param [eventParms]  {Object.<String, String>}
         *
         * @returns {TrackerFlurry}
         */
        timedEventStart: function( eventName, eventParms ) {
            if( !this.inited )
                return this;

            eventParms = eventParms || kEmptyParms;

            FlurryAgent.logEvent( eventName, eventParms, true );

            return this;
        },

        /**
         * End a timed game event
         *
         * @param eventName     {String}
         * @param [eventParms]  {Object.<String, String>}
         *
         * @returns {TrackerFlurry}
         */
        timedEventStop: function( eventName, eventParms ) {
            if( !this.inited )
                return this;

            if( eventParms == null )
                FlurryAgent.endTimedEvent( eventName );
            else
                FlurryAgent.endTimedEvent( eventName, eventParms );

            return this;
        }
    };

    return new TrackerFlurry();
})( document );